/*++

Module Name:

    Backlog.cpp

Abstract:

    Calculating Backlog value, Beta1, Beta2 and Preamble Timings

Environment:

    kernel mode only

Revision History:

    06.09.04 VS: Created

--*/

#ifndef __KERNEL__
#include "stddcls.h"
#else

#define CALC_BACKLOG                0
#define QUICK_TIMING                0

#define BETA1_PRIO_BIT              (1 << 27)
#define COMM_TYPE_SPECIAL_PURPOSE   2
#define PACKET_CYCLE_PORT                            (0x18)
#define BAUD_AND_BIT_SYNC_PORT                       (0x20)
#define BETA2_TICKS_PORT                             (0x2C)
#define PREAMBLE_BITS_PORT                           (0x30)
#define WRITE_RECV_TIME_PORT                         (0x34)

#endif

///////////////////////////////////////////////////////////////////////////////

//  Constants related to Channel Access Algorithm
// maximum size of backlog for access algorithm
#define MAX_BACKLOG 63U

// # of additional slots per backlog increment
#define W_BASE 16U

// Time base for 10 MHz =  600ns -> CT=6
// Time base for 40 MHz =  150ns -> CT=24
// Time base for  5 MHz = 1200ns -> CT=3
#define CT40    24      // in  25 ns
#define CT10    6       // in 100 ns
#define CT5     3       // in 200 ns


///////////////////////////////////////////////////////////////////////////////
#ifndef __KERNEL__
#pragma LOCKEDCODE
#else
#include <linux/random.h>
#endif

/*****************************************************************
*int rand() - returns a pseudo-random number 0 through 32767.
*int Rand() - returns a pseudo-random number 0 through 65535.
*******************************************************************************/

//int __cdecl rand(void)
ULONG Rand(void)
    {
#ifndef __KERNEL__
    static ULONG holdrand = 1;
    // KeQueryTickCount()
    // KeQueryPerformanceCounter(NULL)
    holdrand += (ULONG)KeQueryInterruptTime();
    return ((holdrand = holdrand * 214013L + 2531011L) >> 16);  // & 0x7fff;
#else
  #if LINUX_VERSION_CODE > KERNEL_VERSION(3,8,0) 
    return (prandom_u32() % 65535);
  #else
    return (random32() % 65535);
  #endif
#endif
    }


VOID InitBacklog(PDEVICE_EXTENSION pdx)
    {
    DWORD MsTime = 0; //GetRealMsTime();
//#warning disabled function: InitBacklog
//#ifndef __KERNEL__
    WRITE_IO_ULONG(PACKET_CYCLE_PORT,   pdx->PacketcycleTicks);

    WRITE_IO_ULONG(BETA2_TICKS_PORT,  pdx->beta2TicksFpga);

    // aufrunden, es werden nur Bit 1..9 benutzt
#if QUICK_TIMING
    WRITE_IO_ULONG(PREAMBLE_BITS_PORT,  0);
#else
    WRITE_IO_ULONG(PREAMBLE_BITS_PORT,  pdx->PreambleBits+1);
#endif
    // Bit 0..2 Baud, Bit 4..7 Bit Sync treshold
    WRITE_IO_ULONG(BAUD_AND_BIT_SYNC_PORT,  pdx->Config.comm_clock |
//        ((pdx->Config.params.dir_params.bit_sync_threshold + 4) << 4));
        ((pdx->Config.params.dir_params.bit_sync_threshold + 3) << 4));
#if CALC_BACKLOG
    pdx->backlog = 1;
#endif

    WRITE_IO_ULONG(WRITE_RECV_TIME_PORT,    MsTime);  // 60*60*1000);
    /*DrvPrint(("InitBacklog(): LocalTime=%02d:%02d:%02d.%03d\n",
        MsTime / (60*60*1000),
        MsTime / (   60*1000) % 60,
        MsTime / (      1000) % 60,
        MsTime % 1000
        ));*/
#if defined DBG
#if DBG
    ULONG TimeStamp = READ_IO_ULONG(CURR_RECV_TIME_PORT);
    DrvPrint(("InitBacklog(): TimeStamp=%02d:%02d:%02d.%03d\n",
        TimeStamp / (60*60*1000),
        TimeStamp / (   60*1000) % 60,
        TimeStamp / (      1000) % 60,
        TimeStamp % 1000
        ));
#endif
#endif
//#endif
    }


#if CALC_BACKLOG
/*****************************************************************
Function:  IncrementBacklog
Returns:
Reference:
Purpose:   increments backlog and handles backlog overflow
Comments:
******************************************************************/

VOID IncrementBacklog(PDEVICE_EXTENSION pdx, BYTE deltaBacklog)
    {
//    if (deltaBacklog > MAX_BACKLOG)  return; // Erroneous deltaBacklog value

    // spmGbl.backlog size must allow 2 * MAX_BACKLOG without wrap around
    pdx->backlog += deltaBacklog;
    if (pdx->backlog > MAX_BACKLOG)
        {
        pdx->backlog = MAX_BACKLOG;
        INCR_STATS(pdx->Status.network.backlog_ovflo);
        }
    }


/*****************************************************************
Function:  DecrementBacklog
Returns:
Reference:
Purpose:   decrements backlog and handles backlog underflow
Comments:
******************************************************************/

VOID DecrementBacklog(PDEVICE_EXTENSION pdx, BYTE deltaBacklog)
    {
//    if (deltaBacklog > MAX_BACKLOG)  return; // Erroneous deltaBacklog value

    if (pdx->backlog <= deltaBacklog)
        {
        pdx->backlog = 1;
        }
    else
        {
        pdx->backlog -= deltaBacklog;
        }
    }
#endif


VOID CalculateRecvBacklog(PDEVICE_EXTENSION pdx, BYTE lpduHeader)
    {
#if CALC_BACKLOG
    BYTE deltaBacklog   = LOBYTE(LOWORD(READ_IO_ULONG(DBACKLOG_PORT)));
    BYTE Backlog        = LOBYTE(LOWORD(READ_IO_ULONG(BACKLOG_PORT)));
    DecrementBacklog(pdx, deltaBacklog);

    if ( ((LPDUHeader*)&lpduHeader)->deltaBL == 0 )
        DecrementBacklog(pdx, 1);
    else
        IncrementBacklog(pdx, ((LPDUHeader*)&lpduHeader)->deltaBL);
    DrvPrint(("RecvBacklog(): deltaBacklog=%2d BL=%2d backlog=%2d deltaBL=%d\n",
        deltaBacklog, Backlog, pdx->backlog, ((LPDUHeader*)&lpduHeader)->deltaBL));
#endif
    }

VOID CalculateSendBacklog(PDEVICE_EXTENSION pdx, BYTE lpduHeader,
                          PULONG Beta1RecvTicks, PULONG Beta1SendTicks)
    {
#if CALC_BACKLOG
    BYTE deltaBacklog   = LOBYTE(LOWORD(READ_IO_ULONG(DBACKLOG_PORT)));
    BYTE Backlog        = LOBYTE(LOWORD(READ_IO_ULONG(BACKLOG_PORT)));

    DecrementBacklog(pdx, deltaBacklog);
#endif

    ULONG Beta2Slots;
    if ( ((LPDUHeader*)&lpduHeader)->priority && pdx->Config.node_priority)
        {
        // Priority slots von 1..4..127
        Beta2Slots = pdx->Config.node_priority;
        }
    else
        {
        // Randomizing Slots von 1..1008
        // wir rechnen aber von 0..1007
        // deshalb +1
        Beta2Slots = pdx->Config.channel_priorities +1;
//        Beta2Slots += Rand() % (pdx->backlog * W_BASE);
        }
#if QUICK_TIMING
    *Beta1RecvTicks = 1;
    *Beta1SendTicks = 1;
#else
    *Beta1RecvTicks = pdx->beta1PostRxTicks + pdx->beta2Ticks * Beta2Slots;
    *Beta1SendTicks = pdx->beta1PostTxTicks + pdx->beta2Ticks * Beta2Slots;
#endif
//    *Beta1SendTicks += 65*40;  // Bugfix


#if CALC_BACKLOG
    if ( ((LPDUHeader*)&lpduHeader)->deltaBL == 0 )
        DecrementBacklog(pdx, 1);
    else
        IncrementBacklog(pdx, ((LPDUHeader*)&lpduHeader)->deltaBL);

  #if 0
    DWORD LastBetaValue = READ_IO_ULONG(PREAMBLE_BITS_PORT);

    DrvPrint(("SendBacklog(): LastBetaValue=%6d  << 5=%6d =%6d us\n",
        LastBetaValue, LastBetaValue << 5, (LastBetaValue << 5) /40));
  #endif
    DrvPrint(("SendBacklog(): deltaBacklog=%2d BL=%2d backlog=%2d  Beta1RecvTicks=%6d Beta1SendTicks=%6d\n",
        deltaBacklog, Backlog, pdx->backlog, *Beta1RecvTicks, *Beta1SendTicks));
#endif

    if ( ((LPDUHeader*)&lpduHeader)->priority && pdx->Config.node_priority)
        {
        *Beta1RecvTicks |= BETA1_PRIO_BIT;
        *Beta1SendTicks |= BETA1_PRIO_BIT;
        }
    }


VOID CalculateTimingParameters(PDEVICE_EXTENSION pdx)
    {
//    BREAKPOINT;
    /* Cycle Timer only runs when the Mac Layer is Idle that is when
       the node is not tranmitting or receiving or waiting to transmit
       or counting down beta1 or counting down priority slots or counting
       down the extra 16 beta2 slots after a transmission
       This directly corresponds to the Phase RANDOM_IDLE

       Anytime the MAC layer is busy ie in some phase other than RANDOM_IDLE
       then the cycle timer stops. The cycle timer will then resume when the
       mac layer returns to the RANDOM_IDLE phase. The cycle timer is reset when
       spmGbl.cycleTimerRestart == TRUE. This is set to true
       1) following a successful transmission
       2) following a valid crc receive
    */
    DWORD rxPad;
    DWORD txPad;
    ULONG Hz;
    ULONG usPreambleLen;

    pdx->PacketcycleTicks = (pdx->Config.comm_type == COMM_TYPE_SPECIAL_PURPOSE ? 1794 : 1675)
        * pdx->Config.packet_cycle * CT40;

    rxPad = pdx->Config.recv_interpacket < 128
        ? pdx->Config.recv_interpacket * 41
        : (pdx->Config.recv_interpacket-128) * 145;
    txPad = pdx->Config.xmit_interpacket < 128
        ? pdx->Config.xmit_interpacket * 41
        : (pdx->Config.xmit_interpacket-128) * 145;
    rxPad += pdx->Config.comm_type == COMM_TYPE_SPECIAL_PURPOSE ? 602 : 565;
    txPad += pdx->Config.comm_type == COMM_TYPE_SPECIAL_PURPOSE ? 624 : 583;
    pdx->beta1PostRxTicks = rxPad * CT40;
    pdx->beta1PostTxTicks = txPad * CT40;

    pdx->beta2Ticks     = (pdx->Config.beta2_control + 2UL) * 20 * CT40;
    // beta2TicksFpga auf 8 Bit begrenzen
#if QUICK_TIMING
    pdx->beta2TicksFpga = 1;
#else
    pdx->beta2TicksFpga = MIN(pdx->Config.beta2_control + 2UL, ((ULONG)(0xFF)));
#endif

    Hz = 40000000UL >> (CLOCK_40000 - pdx->Config.input_clock);
    Hz = Hz >> (pdx->Config.comm_clock + 3);
    usPreambleLen = (219 + 32*pdx->Config.preamble_length) * CT10; // in 100 ns
    pdx->PreambleBits = ((Hz * usPreambleLen) / 10000000UL) -1; // counts only Bit Sync
    if (pdx->PreambleBits < 5)  pdx->PreambleBits = 5;          // min 5x "1" + 1x "0"

    dbg("CalculateTimingParameters(): backlog=%d  beta1PostRxTicks=%6u beta1PostTxTicks=%6u beta2Ticks=%u PacketcycleTicks=%u PreambleBits=%u",
        pdx->backlog, ((unsigned int)(pdx->beta1PostRxTicks)), ((unsigned int)(pdx->beta1PostTxTicks)),
        ((unsigned int)(pdx->beta2Ticks)), ((unsigned int)(pdx->PacketcycleTicks)), ((unsigned int)(pdx->PreambleBits)));

    InitBacklog(pdx);
    }

#ifndef __KERNEL__
#pragma LOCKEDCODE                              // force inline functions into nonpaged code
#endif
