/*****************************************************************************/
/*
 * Linux driver for Easylon USB Interfaces
 *
 * Copyright (c)2003 Gesytec GmbH, Volker Schober (info@gesytec.de)
 * Licensend for use with Easylon USB Interfaces by Gesytec
 *
 * Please compile lonusb.c
 *
 */
 /*****************************************************************************/

#define LON_TIMEOUT	300

#define MAX_INTEL_HEX_RECORD_LENGTH 16
typedef struct _INTEL_HEX_RECORD {
	__u16   length;
	__u16   address;
	__u8    type;
	__u8 data[MAX_INTEL_HEX_RECORD_LENGTH];
} INTEL_HEX_RECORD, *PINTEL_HEX_RECORD;

#define FIRMWARE_LOADER	"gesytec/loader.fw"
#define FIRMWARE_FX2	"gesytec/lvu_fx2.fw"

MODULE_FIRMWARE(FIRMWARE_LOADER);
MODULE_FIRMWARE(FIRMWARE_FX2);

#define ANCHOR_LOAD_INTERNAL	0xA0	/* Vendor specific request code for Anchor Upload/Download (This one is implemented in the core) */
#define ANCHOR_LOAD_EXTERNAL	0xA3	/* This command is not implemented in the core. Requires firmware */
#define ANCHOR_LOAD_FPGA	    0xA5	/* This command is not implemented in the core. Requires firmware. Emagic extension */
#define MAX_INTERNAL_ADDRESS	0x1B3F	/* This is the highest internal RAM address for the AN2131Q */
#define CPUCS_REG		        0x7F92	/* EZ-USB Control and Status Register.  Bit 0 controls 8051 reset */
#define CPUCS_REG_FX2           0xE600
#define INTERNAL_RAM(address)   (address <= MAX_INTERNAL_ADDRESS)

#define VR_RESET_NEURON     0xAA    // reset neuron
#define VR_FW_VERSION       0xAB    // Version of LPU firmware
#define VR_FPGA             0xB0
#define VR_FPGA_REG         0xB1    // FPGA register (DR_VendorCmnd)
#define VR_EEPROM             0xA2  // EEPROM upload (DR_VendorCmnd)
#define VR_LATTICE          0xB2    // Lattice device programming commands

// thanks to drivers/usb/serial/keyspan_pda.c code
static int ezusb_writememory(struct usb_device *dev, int addr, const unsigned char *data, int len, __u8 request, __u8 cmp)
{
	int result;
//    dbg("%s(): addr=%04X data=%p len=%d request=%02X", __FUNCTION__, addr, data, len, request);
	unsigned char *buf = kmemdup(data, len, GFP_KERNEL);
	if (!buf)
		return -ENOMEM;

/* Note: usb_control_msg returns negative value on error or length of the
 *               data that was written! */
	result = usb_control_msg(dev, usb_sndctrlpipe(dev, 0), request, 0x40, addr, 0, buf, len, LON_TIMEOUT);
	if (cmp && result >= 0)
        {
		int i;
        //result = usb_control_msg(dev, usb_rcvctrlpipe(dev, 0), request, 0xc0, addr, 0, buf, len, LON_TIMEOUT);

        if (result != len)
            {
            dbg("%s(): An error occurred", __FUNCTION__);
            result  = -1;
            goto out;
            }
        for (i = 0; i < len; i++)
            {
            if (buf[i] != data[i])
                {
                dbg("%s(): %04X: buffer[%d]=%02X != data[%d]=%02X", __FUNCTION__, addr, i, buf[i], i, data[i]);
                result  = -1;
                }
            }
        }
out:
	kfree(buf);
	return result < 0 ? result : 0;
}

static int ezusb_set_reset(struct usb_device *dev, unsigned char reset_bit)
    {
    // toggle the EZ-USB reset bit (harmless on FX2)
    int ret = ezusb_writememory(dev, CPUCS_REG, &reset_bit, 1, 0xa0, 0);
    if (ret < 0)
        {
        dbg("ezusb: set_reset (%d) failed", reset_bit);
        }
    // toggle the FX2 reset bit (harmless on EZ-USB)
    ret = ezusb_writememory(dev, CPUCS_REG_FX2, &reset_bit, 1, 0xa0, 0);
    if (ret < 0)
        {
        dbg("ezusb: set_reset (%d) failed", reset_bit);
        }
    return ret;
    }

static int ezusb_load_firmware(struct usb_device *dev, const void *fw)
{
	const INTEL_HEX_RECORD *hexRecord = fw;
	const struct firmware *loader_fw;
	const INTEL_HEX_RECORD *loader_hx;
	int i;

	int err = request_firmware(&loader_fw, FIRMWARE_LOADER, &dev->dev);
	if (err < 0) 
    {
		dev_err(&dev->dev, "Failed to load %s: %d\n", FIRMWARE_LOADER, err);
		return err;
	}

	// Assert reset (stop the CPU in the ezusb)
	err = ezusb_set_reset(dev, 1);
	if (err < 0)
        {
        dbg("%s - error(1) loading firmware: error = %d", __FUNCTION__, err);
		goto err_fw;
        }
	loader_hx = (const INTEL_HEX_RECORD *)loader_fw->data;
	// 1. We need to put the loader for the firmware into the EZ-USB
	for (i = 0; loader_hx[i].type == 0; i++)
        {
		err = ezusb_writememory(dev, le16_to_cpu(loader_hx[i].address), loader_hx[i].data, le16_to_cpu(loader_hx[i].length), ANCHOR_LOAD_INTERNAL, 0);
        if (err < 0)
            {
            dbg("%s - error(2) loading firmware: error = %d", __FUNCTION__, err);
			goto err_fw;
            }
        }
	// De-assert reset (let the CPU run)
	err = ezusb_set_reset(dev, 0);
	if (err < 0)
        {
        dbg("%s - error(3) loading firmware: error = %d", __FUNCTION__, err);
		goto err_fw;
        }

	// 2. We put the part of the firmware that lies in the external RAM into the EZ-USB
	for (i = 0; hexRecord[i].type == 0; i++)
        {
        if (!INTERNAL_RAM(le16_to_cpu(hexRecord[i].address)))
            {
            err = ezusb_writememory(dev, le16_to_cpu(hexRecord[i].address), hexRecord[i].data, le16_to_cpu(hexRecord[i].length), ANCHOR_LOAD_EXTERNAL, 0);
            if (err < 0)
                {
                dbg("%s - error(4) loading firmware: error = %d", __FUNCTION__, err);
				goto err_fw;
                }
            }
        }
	// Assert reset (stop the CPU in the ezusb)
	err = ezusb_set_reset(dev, 1);
	if (err < 0)
        {
        dbg("%s - error(5) loading firmware: error = %d", __FUNCTION__, err);
		goto err_fw;
        }
	for (i = 0; hexRecord[i].type == 0; i++)
        {
        if (INTERNAL_RAM(le16_to_cpu(hexRecord[i].address)))
            {
            err = ezusb_writememory(dev, le16_to_cpu(hexRecord[i].address), hexRecord[i].data, le16_to_cpu(hexRecord[i].length), ANCHOR_LOAD_INTERNAL, 0);
            if (err < 0)
                {
                dbg("%s i=%d - error(6) loading firmware: error = %d", __FUNCTION__, i, err);
				goto err_fw;
                }
            }
        }

	err = ezusb_set_reset(dev, 0);
	if (err < 0)
        {
		dbg("%s - error(7) loading firmware: error = %d", __FUNCTION__, err);
		goto err_fw;
        }

	err = 0;
err_fw:
	release_firmware(loader_fw);
	return err;
    }

static int Ezusb_VendorRequest(struct usb_device *dev,
                          __u8 request,
                          WORD    wValue,
                          WORD    wIndex,
                          int length,
                          BOOLEAN direction,
                          PBYTE   data)
{
        int result = 0;
        __u16 addr = wValue;
        __u16 len = length;
        //int i;
        unsigned char *buffer = kmalloc(length, GFP_KERNEL);

        if (!buffer) {
                dbg("ezusb: kmalloc(%d) failed.", length);
                return -ENOMEM;
        }

        if (direction)
        {
        result = usb_control_msg(dev, usb_rcvctrlpipe(dev, 0), request, 0xc0, addr, wIndex, buffer, len, LON_TIMEOUT);
        if (result > 0)
          {
            memcpy(data, buffer, length);
          }
        }

        else
        {
        memcpy(buffer, data, length);


/* Note: usb_control_msg returns negative value on error or length of the
 *               data that was written! */
        result = usb_control_msg(dev, usb_sndctrlpipe(dev, 0), request, 0x40, addr, wIndex, buffer, len, LON_TIMEOUT);

        /*if (result >= 0)
        {
        result = usb_control_msg(dev, usb_rcvctrlpipe(dev, 0), request, 0xc0, addr, wIndex, buffer, len, LON_TIMEOUT);

        if (result != len)
            {
            dbg("%s(%04X): An error occurred (result = %i)", __FUNCTION__, ((unsigned int)(request)), result);
            result  = -1;
            goto out;
            }

        for (i = 0; i < len; i++)
            {
            if (buffer[i] != data[i])
                {
                dbg("%s(): %04X: buffer[%d]=%02X != data[%d]=%02X", __FUNCTION__, addr, i, buffer[i], i, data[i]);
                result  = -1;
                }
            }
        }*/
        }
//out:
        kfree(buffer);
        return result;
    }

ULONG ReadFpgaRegister (struct usb_device *dev, ULONG RegIndex)
    {
    BYTE buffer[8];
    ULONG ret;

    memset (buffer, 0x00, sizeof(buffer));

    ret = Ezusb_VendorRequest(dev, VR_FPGA_REG, (BYTE)RegIndex, 0x0000, sizeof(buffer), 1, buffer);

    dbg ("Ezusb_VendorRequest(VR_FPGA_REG)=%X buffer=%02X %02X %02X %02X %02X %02X %02X %02X", ((unsigned int)(ret)),
        ((unsigned int)(buffer[0])), ((unsigned int)(buffer[1])), ((unsigned int)(buffer[2])), ((unsigned int)(buffer[3])),
        ((unsigned int)(buffer[4])), ((unsigned int)(buffer[5])), ((unsigned int)(buffer[6])), ((unsigned int)(buffer[7])));

    if (ret != 5 || buffer[0] != RegIndex)  return -1;

    ret = le32_to_cpu(*((PULONG)&buffer[1]));

    return ret;
    }

VOID WriteFpgaRegister(struct usb_device *dev, ULONG RegIndex, ULONG Value)
    {
    BYTE buffer[8];
    ULONG ret;

    memset (buffer, 0x00, sizeof(buffer));
    buffer[0] = (BYTE)RegIndex;
    *((PULONG)(buffer+1)) = cpu_to_le32(Value);

    ret = Ezusb_VendorRequest(dev, VR_FPGA_REG, RegIndex, 0x0000, sizeof(buffer),  0, buffer);
    }

BOOLEAN EepromAccess(struct usb_device *dev,
                     PVOID               pBuffer,
                     WORD                offset,
                     WORD                ByteCount,
                     BOOLEAN             bReadOperation)
    {
    PBYTE buffer = (PBYTE)pBuffer;
    ULONG ret = Ezusb_VendorRequest(dev,
        //Req      Value   Index   Len        1=in
        VR_EEPROM, offset, 0x0000, ByteCount, bReadOperation, buffer);
    dbg ("Ezusb_VendorRequest(VR_EEPROM)=%X buffer=%02X %02X %02X %02X %02X %02X %02X %02X", ((unsigned int)(ret)),
        ((unsigned int)(buffer[0])), ((unsigned int)(buffer[1])), ((unsigned int)(buffer[2])), ((unsigned int)(buffer[3])),
        ((unsigned int)(buffer[4])), ((unsigned int)(buffer[5])), ((unsigned int)(buffer[6])), ((unsigned int)(buffer[7])));

    return ret != 0;
    }
