/*****************************************************************************/
/*
 * Linux driver for Easylon USB VNI Interfaces
 *
 * Copyright (c)2003 Gesytec GmbH, Volker Schober (info@gesytec.de)
 *
 */
 /*****************************************************************************/

#include "lonusb.h"

#include "dynbuf.c"

#include "driverdata.c"
#include "proc.c"
#include "ezusb.c"
#include "readwrite.c"
#include "fpga.c"
#include "driverentry.c"

#include "XO2_api.cpp"
#include "XO2_cmds.cpp"
#include "XO2_dev.cpp"


/*-------------------------------------------------------------------*/
/**
 *      lvu_debug_data
 */
static inline void lvu_debug_data(int DebugZone, const char *function, int size, const unsigned char *data)
    {
    int i;

    if (!(debug & DebugZone))
        return;

//    printk(KERN_DEBUG __FILE__": %s - length = %d, data = ", function, size);
    printk(KERN_DEBUG "%s ", function);
    for (i = 0; i < size; ++i)
        {
        printk("%.2x ", data[i]);
        }
    printk("\n");
    }

static inline void dump_urb(struct urb * purb)
    {
	dbg("urb                   :%p", purb);
    dbg("dev                   :%p", purb->dev);
	dbg("pipe                  :%08X", purb->pipe);
	dbg("status                :%d", purb->status);
	dbg("transfer_flags        :%08X", purb->transfer_flags);
	dbg("transfer_buffer       :%p", purb->transfer_buffer);
	dbg("transfer_buffer_length:%d", purb->transfer_buffer_length);
	dbg("actual_length         :%d", purb->actual_length);
	dbg("setup_packet          :%p", purb->setup_packet);
	dbg("start_frame           :%d", purb->start_frame);
	dbg("number_of_packets     :%d", purb->number_of_packets);
	dbg("interval              :%d", purb->interval);
	dbg("error_count           :%d", purb->error_count);
	dbg("context               :%p", purb->context);
	dbg("complete              :%p", purb->complete);
    }
/*-------------------------------------------------------------------*/


/**
 *      lvu_delete
 */
static inline void lvu_delete(PDEVICE_EXTENSION dev)
{
    int i;
    int dev_offset = dev->minor * MAX_DEVICENODES;
    
    //printk(KERN_DEBUG "lvu_delete(dev=%p): pSubDev=%p SubDevices=%p\n", dev, dev->pSubDev, dev->SubDevices);

    //if (dev->pSubDev)
    {
        for (i = 0; i < MAX_DEVICENODES; i++)
        {
            qexit(&dev->pSubDev[i].ReadBufferQueue);
            
            device_destroy(lvu_dev_class, first_device + dev_offset + i);
        }
        
        //kfree(dev->pSubDev);
    }

//    if (dev->SubDevices)
//        kfree(dev->SubDevices);
    kfree(dev);
    
    minor_table[dev->minor] = NULL;
    if (dev->intr_in_buff != NULL)
        kfree(dev->intr_in_buff);
    if (dev->bulk_out_buffer != NULL)
        kfree(dev->bulk_out_buffer);
    if (dev->write_urb != NULL)
    {
        dbg("usb_free_urb(dev->write_urb)");
        usb_free_urb(dev->write_urb);
    }
    if (dev->read_urb != NULL)
    {
        dbg("usb_free_urb(dev->read_urb)");
        usb_free_urb(dev->read_urb);
    }
    
    dev->RemainDataSize = 0;
}


/**
 *      lvu_open
 */
static int lvu_open(struct inode *inode, struct file *file)
    {
    int subminor;
    int subdev;
    int submajor;
    int retval = 0;
    PDEVICE_EXTENSION dev = NULL;

    dbgz(DEBUG_OPEN_CLOSE, "%s(): MINOR=%d", __FUNCTION__, MINOR(inode->i_rdev));

    // fix minor_table index
    submajor = imajor(inode);
/*
#ifdef CONFIG_USB_DYNAMIC_MINORS
    // see minor_base at http://www.makelinux.net/ldd3/chp-13-sect-4  
    subminor = iminor(inode);
#else
    subminor = iminor(inode) - lvu_class.minor_base;
#endif
*/
    if (iminor(inode) >= lvu_class.minor_base)
      {
        subminor = iminor(inode) - lvu_class.minor_base;
      }
    else
      {
        subminor = iminor(inode);
      }
    dbg("subminor = %i", subminor);

    if (submajor == MAJOR(first_device))
    {
      subdev = subminor / MAX_DEVICENODES;

      dbg("attempt to open device: (%d, lvu_%d) %d:%d", submajor, subminor, subdev, subminor % MAX_DEVICENODES);

      subminor = subminor % MAX_DEVICENODES;
    }
    else
    {
      err("usb device entry disabled. use standard character devices. current major number is %d", MAJOR(first_device));

      return -ENODEV;
    }

    if ((subdev   < 0) || (subdev   >= MAX_DEVICES) ||
        (subminor < 0) || (subminor >= MAX_DEVICENODES))
        return -ENODEV;

    dbg("minor_table lookup");
    /* lock our minor table and get our local data for this minor */
    mutex_lock(&minor_table_mutex);
    dev = minor_table[subdev];
    if (dev == NULL) {
        mutex_unlock(&minor_table_mutex);
        return -ENODEV;
        }

    if (0 < dev->pSubDev[subminor].open_count)
    {
      mutex_unlock(&minor_table_mutex);

      return -EBUSY;
    }

    dbg("minor_table lookup ok");

    mutex_unlock(&minor_table_mutex);

    /* lock this device */
	mutex_lock(&dev->mutex);
    /* unlock the minor table */
    dbg("usage count test ok");

    if (retval)
        {
        return -ERESTARTSYS;
        }

    /* increment our usage count for the driver */
    ++(dev->pSubDev[subminor].open_count);

    ++dev->sub_open_count;

    /* save our object in the file's private structure */
    file->private_data = &(dev->pSubDev[subminor]);
    dev->pSubDev[subminor].DevStats.open++;


    dev->RemainDataSize = 0;
    qflush(&dev->pSubDev[subminor].ReadBufferQueue);

    if (1 == dev->sub_open_count)
    {
#if 0
        dev->read_urb->status = 0;	    // needed!
        dev->read_urb->dev    = dev->usbdev;	// needed!
#else
        dev->read_urb->status = 0;	    // needed!
        {
            int pipe = usb_rcvintpipe(dev->usbdev, dev->intr_in_ep_addr);
            usb_fill_int_urb(dev->read_urb, dev->usbdev, pipe,
                dev->intr_in_buff, dev->bulk_in_size,
                lvu_read_int_callback, dev,
                dev->bInterval);
        }
#endif
        dev->write_urb->dev   = dev->usbdev;	// needed!

        reset_neuron(dev, FALSE, TRUE, POWER_UP_RESET);

        dbg("usb_submit_urb(read_urb=%p intr_in_ep_addr=%02X):", dev->read_urb, dev->intr_in_ep_addr);
        dump_urb(dev->read_urb);
        dev->DevStats.submit_read1++;
        retval = USB_SUBMIT_URB(dev->read_urb, GFP_KERNEL);
        if (retval)
        {
            dev->DevStats.submit_read1_error++;
            err("%s(): usb_submit_urb(read_urb=%02X)=%d", __FUNCTION__, dev->intr_in_ep_addr, retval);
        }
        else
            dev->DevStats.submit_read1_ok++;
        
        dump_urb(dev->read_urb);
        
        if (retval < 0)
        {
            /* decrement our usage count for the device */
            --dev->sub_open_count;
            --(dev->pSubDev[subminor].open_count);
        }
    }

    /* unlock this device */
	mutex_unlock(&dev->mutex);

    dbg("%s(): retval=%d open_count=%d", __FUNCTION__, retval, dev->sub_open_count);
    return retval;
    }


/**
 *      lvu_release
 */
static int lvu_release(struct inode *inode, struct file *file)
{
    PSUB_DEVICE_EXTENSION dev = (PSUB_DEVICE_EXTENSION)file->private_data;
    int retval = 0;
    
    if (dev == NULL) 
    {
        dbg("%s() object is NULL", __FUNCTION__);
        return -ENODEV;
    }
    
    dbgz(DEBUG_OPEN_CLOSE, "%s(): minor=%d open_count=%d (%d)", __FUNCTION__, (dev->udev->minor * MAX_DEVICENODES) + dev->SubdevNum, dev->open_count, dev->udev->sub_open_count);
    
    /* lock our minor table */
    mutex_lock(&minor_table_mutex);
    
    /* lock our device */
    mutex_lock(&dev->udev->mutex);
    
    if (dev->open_count <= 0) 
    {
        dbg("%s() device not opened", __FUNCTION__);
        retval = -ENODEV;
        goto exit_not_opened;
    }
    
    if ((dev->udev->usbdev == NULL) && (1 >= dev->udev->sub_open_count)) 
    {
        /* the device was unplugged before the file was released */
        mutex_unlock(&dev->udev->mutex);
        lvu_delete(dev->udev);
        mutex_unlock(&minor_table_mutex);
        return 0;
    }
    
    --(dev->udev->sub_open_count);
    
    --dev->open_count;
    
    /* decrement our usage count for the device */
    if (dev->udev->sub_open_count <= 0)
    {
        /* shutdown any bulk writes that might be going on */
        dump_urb(dev->udev->read_urb);
        usb_kill_urb(dev->udev->read_urb);
        dump_urb(dev->udev->read_urb);
        
        usb_kill_urb(dev->udev->write_urb);
        
        dev->udev->RemainDataSize = 0;
        qflush(&dev->ReadBufferQueue);
        dev->open_count = 0;
        
        // if (usb_reset_configuration(dev->usbdev) < 0)
        //     err("usb_reset_configuration() failed");
    }
    
    
exit_not_opened:
    mutex_unlock(&dev->udev->mutex);
    mutex_unlock(&minor_table_mutex);
    dev->DevStats.close++;
    
    return retval;
}
