/****************************************************************************
 *
 *  FILE DESCRIPTION:  This file contains platform dependant flags
 *  and basic data types.  All data types of NI, NS, as well as internal
 *  data types used by the NSS core code are derived from the basic
 *  data types of this file.
 *
 *  If you are porting the NI or NS layer to a non-Win32, non-DOS platform,
 *  please examine the data types defined in this file, and modify them
 *  as appropriate to your OS, compiler, and CPU.
 *
 *  -------- Note on Byte-Ordering -----
 *  The defines BIG_ENDIAN and LITTLE_ENDIAN are used in these files to
 *  indicate the byte ordering.
 *
 *  BIG_ENDIAN means the high byte value is located in low memory address,
 *  and bits are numbered from left to right. The Motorola 68k family and
 *  the Neuron Chip are examples of BIG_ENDIAN processors.
 *
 *  LITTLE_ENDIAN means the high byte value is located in high memory address,
 *  and bits are numbered from right to left. The Intel 80x86 processors are
 *  an example of LITTLE_ENDIAN processors.
 *
 *  The following description of these terms from "C, A Reference Manual"
 *  by Samuel P. Harbison and Guy L. Steele Jr. may refresh your memory:
 *
 *  "Computers differ in their storage byte order; that is, they differ in
 *  which byte of storage they consider to be first in a larger piece.  In
 *  "right-to-left" or "little endian" architectures, which include the
 *  DEC VAX computers and the Intel 80x86 microprocessors, the address of
 *  a 32-bit integer is the address of the low-order byte of the integer.
 *  In "left-to-right" or "big endian" architectures, which include the
 *  Motorola 680x0 microprocessor family and the Zilog Z8000, the address
 *  of a 32-bit integer is the address of the high-order byte of the
 *  integer."
 *
 *  "Components of a structure type are allocated in the order of increasing
 *  addresses, that is, either left to right or right to left depending
 *  on the byte order of the computer.  Because bit fields are also packed
 *  following the byte order, it is natural to number the bits in a
 *  piece of storage following the same convention.  Thus, in a left-to-right
 *  computer the most significant (leftmost) bit of a 32-bit integer would
 *  be bit number 0 and the least significant bit would be bit number 31.
 *  In right-to-left computers the least significant (rightmost) bit would
 *  be bit 0, and so forth".
 *
 *  Copyright (c) 1994-1997 Echelon Corporation.  All Rights Reserved.
 *
 ***************************************************************************/
#ifndef _NI_PLATFORM_H
#define _NI_PLATFORM_H

/* The following symbols define the platform.
 * You may change them if they are not correct for your environment.
 * See the file ns_opts.h
 */
#include "ns_opts.h"

#if defined(__linux__) && !defined(LINUX)
  #pragma message ("__linux__ defined")
  #define LINUX
#endif

#ifdef __BIG_ENDIAN
  #pragma message ("__BIG_ENDIAN defined")
#endif
#ifdef __LITTLE_ENDIAN
  #pragma message ("__LITTLE_ENDIAN defined")
#endif

#ifdef __BIG_ENDIAN_BITFIELD
  #pragma message ("__BIG_ENDIAN_BITFIELD defined")
#endif
#ifdef __LITTLE_ENDIAN_BITFIELD
  #pragma message ("__LITTLE_ENDIAN_BITFIELD defined")
#endif


#ifdef LINUX
  #ifdef __LITTLE_ENDIAN
    #undef BIG_ENDIAN
    #ifndef LITTLE_ENDIAN
      #define LITTLE_ENDIAN
    #endif
  #else
    #undef LITTLE_ENDIAN
    #ifndef BIG_ENDIAN
      #define BIG_ENDIAN
    #endif
  #endif
  #define NS_CDECL
#else

  #ifndef BIG_ENDIAN
    #ifndef LITTLE_ENDIAN
      /* ENDIAN not specified, use default settings */

      /* Neuron-C programs.  The neuron-C compiler automatically defines
         _ECHELON as the result of including echelon.h */
      #ifdef _ECHELON
        #define NS_CDECL
        #define NEURON_HOSTED               /* runs on a Neuron Chip */
        #define BIG_ENDIAN                  /* neuron is always BIG_ENDIAN */
        #define COMPILE_8                   /* 8-bit compiler */
        /* Use this to substitute an expression based on platform */
        #define NEURON_OR_WIN32(neuron_exp,win32_exp) neuron_exp

/* Win32 programs.  By default, Microsoft Visual C++ includes WIN32 as the
   project setting.  If your compiler doesn't do that, you should manually
   specify it to the compiler.

   This section also assumes that Win32 runs on a LITTLE_ENDIAN machine,
   which is the typical case.  If you are running Win32 on a BIG_ENDIAN
   machine, be sure to change LITTLE_ENDIAN to BIG_ENDIAN */
      #else
        #ifdef WIN32
          #define NS_CDECL        cdecl
          #define WIN32_HOSTED                /* runs in Win32 OS environment */
		  #define LITTLE_ENDIAN               /* change to BIG_ENDIAN if you need to */
		  /* Use this to substitute an expression based on platform */
		  #define NEURON_OR_WIN32(neuron_exp,win32_exp) win32_exp

/* Other environment -- assume LITTLE_ENDIAN byte and bit orders.  If it is
   not the case, replace it with BIG_ENDIAN and/or BITF_BIG_ENDIAN.  */
        #else
		  #define NS_CDECL
		  #define LITTLE_ENDIAN
		#endif

      #endif  // #ifdef _ECHELON
    #endif  /* ifndef LITTLE_ENDIAN */
  #endif  /* ifndef BIG_ENDIAN */
#endif  // #ifdef LINUX

#ifdef __LITTLE_ENDIAN_BITFIELD
  #pragma message ("Bitfield: Little Endian")
  #define BITF_LITTLE_ENDIAN
#endif
#ifdef __BIG_ENDIAN_BITFIELD
  #pragma message ("Bitfield: Big Endian")
  #define BITF_BIG_ENDIAN
#endif

/* If bitfield ENDIAN not specified, base it on byte field ENDIAN setting
*/
#ifndef BITF_BIG_ENDIAN
  #ifndef BITF_LITTLE_ENDIAN

    #ifdef BIG_ENDIAN
      #define BITF_BIG_ENDIAN
    #else
      #define BITF_LITTLE_ENDIAN
    #endif

  #endif  /* ifndef BITF_LITTLE_ENDIAN */
#endif  /* ifndef BITF_BIG_ENDIAN */

#ifndef NS_CDECL
  #define NS_CDECL
#endif

#ifdef _MSC_VER
  /* Default packing alignment for NSS structures is single byte boundaries */
  #define _NSS_PACKING        1
  #pragma pack(_NSS_PACKING)
#endif

#if defined LINUX
  #pragma pack(1)
#endif


#ifndef LINUX
  #ifdef LITTLE_ENDIAN
    #pragma message ("Neuron: Byte Little Endian")
  #else
    #pragma message ("Neuron: Byte Big Endian")
  #endif

  #ifdef BITF_LITTLE_ENDIAN
    #pragma message ("Neuron: Bitfield Little Endian")
  #endif

  #ifdef BITF_BIG_ENDIAN
    #pragma message ("Neuron: Bitfield Big Endian")
  #endif
#endif

#ifdef __cplusplus
namespace LON_PLATFORM
{
#endif

/* Basic data types for 8-bit compilers such as the neuron-C compiler.
 * "short is assumed to be 8-bit, "long" is assumed to be 16-bit.
 * If that is not the case for your compiler,
 * replace them with another 8 and 16 bit types.
 */
#ifdef COMPILE_8
typedef unsigned short Byte;        /* 8-bits */
typedef unsigned long Word;         /* 16-bits */
#define NetEnum(enumType) enumType  /* 8-bit enums */
#define MAX_NET_ENUM (-1)           /* maximum value for enum */
typedef boolean Bool;               /* 8-bit boolean */
typedef unsigned int BitField;      /* bit field */
typedef struct {int bytes[4];} Long;/* 32-bits */

/* Basic data types for 16 and 32 bit compilers.
 * "short" is assumed to be 16-bit.
 * If that is not the case for your compiler,
 * replace it with another 16-bit type.
 */
#else
#if defined LINUX
  typedef uint8_t Byte;         /* 8-bits */
  typedef uint16_t Word;        /* 16-bits (assumes short to be 16-bits) */
  typedef uint32_t Long;
#else
  typedef unsigned char Byte;         /* 8-bits */
  typedef unsigned short Word;        /* 16-bits (assumes short to be 16-bits) */
  typedef unsigned long Long;
#endif
  #define NetEnum(enumType) Byte      /* 8-bit enums */
  #define MAX_NET_ENUM (255)          /* maximum value for enum */
  typedef Byte Bool;                  /* 8-bit boolean */
                                      /* bit field, bit aligned */
  #ifdef _MSC_VER                     /* Microsoft compiler */
    typedef unsigned char BitField;
  #else                               /* Borland, neuron-C */
    #ifndef LINUX                     /* Borland, neuron-C */
      typedef unsigned int BitField;
    #else
      typedef unsigned char BitField;
    #endif
  #endif
#endif

/* true and false */
#ifndef _ECHELON
#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif
#endif


/* 16-bit word (BIG_ENDIAN ordering if exposed, native ordering if internal) */
typedef Word NetWord;
typedef BitField bits, Bits;
typedef NetWord niWord;
typedef NetWord NeuronMemAddr;              /* neuron memory address */

/* 32-bit word */
  typedef Long NetLong;

#if !defined(_WINNT_) && !defined(_NTDEF_)
  #ifndef WCHAR
    #ifdef _WCHAR_T_DEFINED
      typedef wchar_t WCHAR;                 /* wide character type - for Unicode */
    #else
      typedef Word WCHAR;                 /* wide character type - for Unicode */
    #endif
  #endif
#endif

/* The following section contains short-hands for some frequently-used types.
   They are provided as a convenience.
 */

#if defined LINUX
  typedef uint32_t Uint;          /* unsigned integer, native size and byte order */
  typedef uint32_t Ulong;        /* unsigned long, native size and byte order */
  typedef uint16_t Ushort;      /* unsigned short, native size and byte order */
  typedef uint8_t Uchar;        /* unsigned char */
#else
  typedef unsigned int Uint;          /* unsigned integer, native size and byte order */
  typedef unsigned long Ulong;        /* unsigned long, native size and byte order */
  typedef unsigned short Ushort;      /* unsigned short, native size and byte order */
  typedef unsigned char Uchar;        /* unsigned char */
#endif

#ifdef __cplusplus
}; // namespace LON_PLATFORM
#endif

/* swap bytes if native byte-ordering is different from NSS */
#ifdef LITTLE_ENDIAN
  #define NET_SWAB(aWord)      ( (((unsigned)(aWord))>>8) | (((aWord)&0xff)<<8) )
  #define NET_SWAB_LONG(aLong) ( (((unsigned)(aLong))>>24) | (((aLong)>>8)&0xff00) | \
                                 (((aLong)<<8)&0xff0000) | ((aLong)<<24) )
  #define NET_SWAP_WORD(aWord) aWord = NET_SWAB(aWord)
  /*_swab((char *)&(aWord), (char *)&(aWord), 2) */
  #define NET_SWAP_LONG(aLong) aLong = NET_SWAB_LONG(aLong)
#else
  #define NET_SWAB(aWord) (aWord)
  #define NET_SWAB_LONG(aLong)
  #define NET_SWAP_WORD(aWord)
  #define NET_SWAP_LONG(aWord)
#endif  // #ifdef LITTLE_ENDIAN

#ifdef _MSC_VER
  #pragma pack()
#endif

#if defined LINUX
#pragma pack()
#endif

#ifdef __cplusplus
using namespace LON_PLATFORM;
#endif

#endif  // _NI_PLATFORM_H
