/*****************************************************************************/
/*
 * Linux driver for Easylon USB Interfaces
 *
 * Copyright (c)2003 Gesytec GmbH, Volker Schober (info@gesytec.de)
 * Licensend for use with Easylon USB Interfaces by Gesytec
 *
 * Please compile lonusb.c
 *
 */
 /*****************************************************************************/

#pragma pack(1)

typedef struct _LVU_HEADER
    {
    __u32           TimeStamp;              // in ms (Dauer = 49.7 Tage)
    __u8            Backlog;                // Current Backlog
    __u16           Accuracy;               //
    } LVU_HEADER, *PLVU_HEADER;

typedef struct _LON_MESSAGE_HEADER {
    UCHAR   Id;
    UCHAR   Len;
    } LON_MESSAGE_HEADER, *PLON_MESSAGE_HEADER;

typedef struct _LVU_SEND_HEADER
    {
      __u16          Len;
      __u32          Beta1RecvTicks;
      __u32          Beta1SendTicks;
    } LVU_SEND_HEADER, *PLVU_SEND_HEADER;

#define LNA_DATA_LEN     sizeof(LVU_HEADER)
#define LON_LVU_OFFSET   (sizeof(LVU_HEADER) - sizeof(LON_MESSAGE_HEADER))
#define LON_SEND_OFFSET  (sizeof(LVU_SEND_HEADER) - sizeof(LON_MESSAGE_HEADER))
#pragma pack()


#define BOUNCE_TIME          2            // 2 s Service pin bounce time

#define CRC_ERROR_MASK       0x80
#define SERVICE_PIN_MASK     0x40
#define BACKLOG_MASK         0x3F


#include "backlog.cpp"
#include "neuron.cpp"
#include <linux/time.h>


static int lvu_write_real(PSUB_DEVICE_EXTENSION dev);


#define RetErrnoName(x)  case x: return #x

static inline char *ErrnoName(int ch)
    {
    static char RetKeyName[3] = {0};
    switch (ch)
        {
//        case EOF: return "EOF";
        RetErrnoName(ENOENT);
        RetErrnoName(ECONNRESET);
        RetErrnoName(EILSEQ);
        RetErrnoName(ESHUTDOWN);
        default: return RetKeyName;
        }
    }


/**
 *      lvu_poll
 */
static unsigned lvu_poll( struct file *file, struct poll_table_struct *wait)
{
    PSUB_DEVICE_EXTENSION dev = (PSUB_DEVICE_EXTENSION)file->private_data;
    unsigned mask = 0;


    dev->DevStats.poll++;
    /* verify that the device wasn't unplugged */
    if (dev->udev == NULL || dev->udev->usbdev == NULL || dev->udev->remove_pending)
    {
        dev->DevStats.poll_error++;
        dbg("%s(): device unplugged (minor=%i)", __FUNCTION__, dev->SubdevNum);

        return POLLERR | POLLHUP;
    }

    if (file->f_mode & FMODE_READ)
    {
        dev->DevStats.poll_read++;
        poll_wait(file, &dev->rd_wait, wait);
    }
    if (file->f_mode & FMODE_WRITE)
    {
        dev->DevStats.poll_write++;
        poll_wait(file, &dev->udev->wr_wait, wait);
    }


    /* verify that the device wasn't unplugged */
    if (dev->udev == NULL || dev->udev->usbdev == NULL || dev->udev->remove_pending)
    {
        dev->DevStats.poll_error++;
        return POLLERR | POLLHUP;
    }

    if (qhasdata(&dev->ReadBufferQueue))
	{
		dev->DevStats.poll_pollin++;
        mask |= POLLIN | POLLRDNORM;
    }

    if (dev->udev->write_urb->status != -EINPROGRESS)
    {
		dev->DevStats.poll_pollout++;
        mask |= POLLOUT | POLLWRNORM;
    }

    //dbg("%s(): nach poll_wait() return %X", __FUNCTION__, mask);
    return mask;
}

static void lvu_read_int_callback(struct urb *urb)
{
    PLON_MESSAGE_HEADER pLonH;
    byte buffer [512];
    NTSTATUS status;
    unsigned int TransferLen;
    int ret, i;

    PDEVICE_EXTENSION dev = (PDEVICE_EXTENSION)urb->context;

    dbg("%s(): minor=%d urb->status=%d actual_length=%02X",
        __FUNCTION__, dev->minor, urb->status, urb->actual_length);

    switch (urb->status)
    {
    case 0:
        dev->DevStats.int_callback_ok++;
        /* success */
        break;
    case -ECONNRESET:
    case -ENOENT:
    case -ESHUTDOWN:
        dev->DevStats.int_callback_abort++;
        /* this urb is terminated, clean up */
        dbg("%s(): urb shutting down with status: %d=%s", __FUNCTION__, urb->status, ErrnoName(-urb->status));
        dev->RemainDataSize = 0;
        return;
    default:
        dev->DevStats.int_callback_error++;
        dbg("%s(): nonzero urb status received: %d", __FUNCTION__, urb->status);
        dev->RemainDataSize = 0;
        goto exit;
    }

    lvu_debug_data(DEBUG_DUMP3, "<<<", urb->actual_length, dev->intr_in_buff);
    // dbg("current_frame_number=%d", usb_get_current_frame_number(dev->usbdev));


    TransferLen = urb->actual_length;
    TransferLen += dev->RemainDataSize;
    if (TransferLen == 0)  goto ExitComplete;

    if (TransferLen > sizeof(dev->intdata))
    {
        dbg("!!! TransferLen=%u > sizeof(dev->intdata)=%zu", TransferLen, sizeof(dev->intdata));
        TransferLen = dev->RemainDataSize = 0;
        goto exit;
    }
    memcpy(&dev->intdata[dev->RemainDataSize], dev->intr_in_buff, urb->actual_length);

    if (urb->actual_length == dev->bulk_in_size)
    {
        dev->RemainDataSize = TransferLen;
        dbg("  RemainDataSize=%u:", dev->RemainDataSize);
        goto exit;
    }
    dev->RemainDataSize = 0;

    dbg("TransferLen=%u", TransferLen);

    lvu_debug_data(DEBUG_DUMP2, "<< ", TransferLen, dev->intdata);
    dev->DevStats.usb_in++;

    if (LNA_DATA_LEN < TransferLen)
    {
        memmove(buffer, dev->intdata + LON_LVU_OFFSET, TransferLen - LON_LVU_OFFSET);

        TransferLen -= LON_LVU_OFFSET;

        pLonH = (PLON_MESSAGE_HEADER)(buffer);
        if (TransferLen < 8+2)
        {
            pLonH->Id = (TransferLen-2) == 0 ? niTIMEOUT : niVNI_SHORT;
            INCR_STATS(dev->Status.base.xmit_errors);
        }
        else if (((PLVU_HEADER)(dev->intdata))->Backlog & CRC_ERROR_MASK)
        {
            pLonH->Id = niVNI_BADCRC;
            INCR_STATS(dev->Status.base.xmit_errors);
        }
        else
        {
            pLonH->Id  = niVNI_RECV;
            INCR_STATS(dev->Status.network.node_received);
        }
        pLonH->Len = TransferLen - 2;

        for (i = 0; i < MAX_DEVICENODES; i++)
        {
            if (0 < dev->pSubDev[i].open_count)
            {
                status = qwrite(&dev->pSubDev[i].ReadBufferQueue, buffer, TransferLen, FALSE);

                dbg("qwrite()=%d", status);
            }
        }
    }
    else
    {
        dbg("short message from fpga");

        if (((PLVU_HEADER)(dev->intdata))->Backlog & SERVICE_PIN_MASK)
        {
            TransferLen = 0;

            if ( (get_seconds() - dev->ServicePinTimeStamp) > BOUNCE_TIME)
            {
                dev->ServicePinTimeStamp = get_seconds();

                dbg("send service pin request");

                TransferLen = 2;
                buffer[0] = niSERVICE;
                buffer[1] = 0;

                for (i = 0; i < MAX_DEVICENODES; i++)
                {
                    if (0 < dev->pSubDev[i].open_count)
                    {
                        status = qwrite(&dev->pSubDev[i].ReadBufferQueue, buffer, TransferLen, FALSE);
                        dbg("qwrite()=%d", status);
                    }
                }
            }
        }
    }

ExitComplete:
    for (i = 0; i < MAX_DEVICENODES; i++)
    {
        wake_up_interruptible(&dev->pSubDev[i].rd_wait);
    }
exit:
#if 0
    urb->status = 0;	// needed!
#else
    urb->status = 0;	// needed!
    {
        int pipe = usb_rcvintpipe(dev->usbdev, dev->intr_in_ep_addr);
        usb_fill_int_urb(dev->read_urb, dev->usbdev, pipe,
            dev->intr_in_buff, dev->bulk_in_size,
            lvu_read_int_callback, dev,
            dev->bInterval);
    }
#endif
    dev->DevStats.submit_read2++;
	ret = usb_submit_urb(urb, GFP_ATOMIC);
    if (ret)
    {
        dev->DevStats.submit_read2_error++;
        err("%s(): usb_submit_urb()=%d", __FUNCTION__, ret);
    }
    else
        dev->DevStats.submit_read2_ok++;
}



/**
 *      lvu_read
 */
static ssize_t lvu_read(struct file *file, char __user *buffer, size_t count, loff_t *ppos)
{
    PSUB_DEVICE_EXTENSION dev = file->private_data;
    int ret = 0;

    dbg("%s(): minor=%d:%d, count=%zu", __FUNCTION__, dev->udev->minor, dev->SubdevNum, count);
	dev->DevStats.read_in++;

    if (dev->udev == NULL)
    {
        dbg("%s()=-ENODEV !!!", __FUNCTION__);
        dev->DevStats.read_error++;
        return -ENODEV;
    }

    mutex_lock(&dev->udev->mutex);

    // verify that the device wasn't unplugged
    if (dev->udev->usbdev == NULL || dev->udev->remove_pending)
    {
        mutex_unlock(&dev->udev->mutex);
        dev->DevStats.read_error++;
        dbg("%s()=-ENODEV !!!", __FUNCTION__);
        return -ENODEV;
    }

    if (qempty(&dev->ReadBufferQueue))
    {
        dbg("%s(): qempty", __FUNCTION__);
        mutex_unlock(&dev->udev->mutex);
        dbg("nach up");

        if (file->f_flags & O_NONBLOCK)
        {
            dbg("%s()=-EAGAIN !!!", __FUNCTION__);
            dev->DevStats.read_error++;
            return -EAGAIN;  // -EWOULDBLOCK;
        }

        dbg("%s(): vor wait_event_interruptible(rd_wait)", __FUNCTION__);
        ret = wait_event_interruptible(dev->rd_wait, (qhasdata(&dev->ReadBufferQueue) || dev->udev->remove_pending));
        if (ret < 0)
        {
            dev->DevStats.read_error++;
            return ret;
        }

	    if (dev->udev == NULL)
	    {
            dev->DevStats.read_error++;
	        dbg("%s()=-ENODEV !!!", __FUNCTION__);
	        return -ENODEV;
	    }

        mutex_lock(&dev->udev->mutex);
        if (dev->udev->usbdev == NULL || dev->udev->remove_pending)
        {
            mutex_unlock(&dev->udev->mutex);
            dev->DevStats.read_error++;
            dbg("%s()=-ENODEV !!!", __FUNCTION__);
            return -ENODEV;
        }

    }

    dbg("%s(): vor qhasdata()", __FUNCTION__);

    // if the read was successful, copy the data to userspace
    if (qhasdata(&dev->ReadBufferQueue))
    {
        ret = qread(&dev->ReadBufferQueue, buffer, count, TRUE);

        lvu_debug_data(DEBUG_DUMP1, "<  ", ret, buffer);
    }

    mutex_unlock(&dev->udev->mutex);
    dbg("%s() return %d", __FUNCTION__, ret);
    dev->DevStats.read_ok++;
    return ret;
}


/**
 *      lvu_write_bulk_callback
 */
static void lvu_write_bulk_callback(struct urb *urb)
    {
    PDEVICE_EXTENSION dev = (PDEVICE_EXTENSION)urb->context;

    dbg("%s(): minor %d", __FUNCTION__, dev->minor);

    wake_up_interruptible(&dev->wr_wait);

    if (urb->status != 0)  // USB_ST_NOERROR
        {
        dev->DevStats.write_error++;
        dbg("%s() write bulk error: %d=%s", __FUNCTION__, urb->status, ErrnoName(-urb->status));
        return;
        }
    else
        dev->DevStats.write_ok++;

    dbg("%s() write bulk done", __FUNCTION__);
    return;
    }


/**
 *      lvu_write
 *
 *      - read user data into input gathering buffer (128bytes, usage count)
 *      - move complete packet into send buffer, if send buffer is empty and a full packet is in gathering buffer
 *      - the send buffer has size of 128byte and usage count. on moving a message to send buffer the header and crc
 *        are being added
 *      - send the contents of send buffer with possibly multiple urb
 */
static ssize_t lvu_write(struct file *file, const char __user *buffer, size_t count, loff_t *ppos)
{
    PSUB_DEVICE_EXTENSION dev = (PSUB_DEVICE_EXTENSION)file->private_data;
    ssize_t bytes_written = 0;
    int retval = 0;

    dev->DevStats.write_in++;
    dbg("%s(%d): minor=%d:%d, count=%zu bulk_out_size=%d", __FUNCTION__, dev->SubdevNum, dev->udev->minor, dev->SubdevNum, count, dev->udev->bulk_out_size);


    if (dev->udev == NULL)
    {
        dbg("%s(%d)=-ENODEV !!!", __FUNCTION__, dev->SubdevNum);
        return -ENODEV;
    }

    mutex_lock(&dev->udev->mutex);
    //dbg("%s(%d): mutex_lock1(%p): count=%d", __FUNCTION__, dev->SubdevNum, &dev->udev->mutex, dev->udev->mutex.count);

    // verify that the device wasn't unplugged
    if (dev->udev->usbdev == NULL || dev->udev->remove_pending)
    {
        dbg("%s(%d) dev->usbdev=%p dev->remove_pending=%X", __FUNCTION__, dev->SubdevNum, dev->udev->usbdev, dev->udev->remove_pending);
        retval = -ENODEV;
        goto exit;
    }

    // verify that we actually have some data to write
    if (count == 0)
    {
        dbg("%s(%d) write request of 0 bytes", __FUNCTION__, dev->SubdevNum);
        goto exit;
    }

    while (dev->udev->write_urb->status == -EINPROGRESS)
    {
        dbg("%s(%d): write_urb->status == -EINPROGRESS", __FUNCTION__, dev->SubdevNum);
        mutex_unlock(&dev->udev->mutex);
        if (file->f_flags & O_NONBLOCK)
        {
            dbg("%s(%d)=-EAGAIN !!!", __FUNCTION__, dev->SubdevNum);
            return -EAGAIN;  // -EWOULDBLOCK;
        }

        retval = wait_event_interruptible(dev->udev->wr_wait, dev->udev->write_urb->status != -EINPROGRESS);
        if (retval < 0)
            return retval;

        dbg("%s(%d): nach wait_event_interruptible(wr_wait)", __FUNCTION__, dev->SubdevNum);
        mutex_lock(&dev->udev->mutex);
        //dbg("%s(%d): mutex_lock2(%p): count=%d", __FUNCTION__, dev->SubdevNum, &dev->udev->mutex, dev->udev->mutex.count);
    }

    // we can only write as much as 1 urb will hold
    bytes_written = (count > (dev->udev->bulk_out_size - dev->send_buffer_used)) ? (dev->udev->bulk_out_size - dev->send_buffer_used) : count;

    if (copy_from_user(dev->send_buffer + dev->send_buffer_used, buffer, bytes_written))
    {
        dbg("%s(%d): copy_from_user()=-EFAULT", __FUNCTION__, dev->SubdevNum);
        retval = -EFAULT;
        goto exit;
    }

    //lvu_debug_data(DEBUG_DUMP1, ">  ", bytes_written, dev->write_urb->transfer_buffer);
    lvu_debug_data(DEBUG_DUMP1, ">  ", bytes_written, dev->send_buffer + dev->send_buffer_used);

    dev->send_buffer_used += bytes_written;

    //dbg("%s() send_buffer_used=%d Len=%d", __FUNCTION__, dev->send_buffer_used, ((PLON_MESSAGE_HEADER)(dev->send_buffer))->Len);
    if ((2 <= dev->send_buffer_used) && (dev->send_buffer_used >= ((PLON_MESSAGE_HEADER)(dev->send_buffer))->Len +2))
    {
        retval = lvu_write_real(dev);
    }

exit:
    mutex_unlock(&dev->udev->mutex);
    dbg("%s(%d) return %d", __FUNCTION__, dev->SubdevNum, retval);
    return retval;
}


static int lvu_write_real(PSUB_DEVICE_EXTENSION dev)
{
    PLON_MESSAGE_HEADER  current_src_msg = ((PLON_MESSAGE_HEADER)(dev->send_buffer));
    ssize_t              bytes_written = 0;
    ssize_t              bytes_current = 0;
    BYTE                *current_packet = dev->udev->write_urb->transfer_buffer;
    int                  retval = 0;
    USHORT               crc16;
    PUSHORT              pcrc16;
    byte                 buffer_in [256];
    byte                 buffer_out [256];
    ULONG                TransferLen;
    NTSTATUS             status;
    int                  i;

    while (0 <= bytes_current)
    {
        if ((current_src_msg->Len +2) <= dev->send_buffer_used)
        {
            if (dev->udev == NULL || dev->udev->usbdev == NULL || dev->udev->remove_pending)
            {
                retval = -ESHUTDOWN;
                break;
            }

            if (current_src_msg->Id == niVNI_SEND || current_src_msg->Id == niVNI_RECV || current_src_msg->Id == niVNI_SEND_PRIO)
            {  // VNI
                dbg("got niVNI message");

                if ((current_src_msg->Len + LON_SEND_OFFSET + 4) <= (dev->udev->bulk_out_size - bytes_written))
                {
                    memcpy(buffer_in, current_src_msg, current_src_msg->Len +2);
                    buffer_in[0] = niVNI_RECV;
                    buffer_in[1] += 2;

                    crc16   = CRC16(buffer_in + sizeof(LON_MESSAGE_HEADER), current_src_msg->Len);
                    pcrc16  = (PUSHORT)(buffer_in + current_src_msg->Len + 2);
                    *pcrc16 = NET_SWAB(crc16);

                    bytes_current = current_src_msg->Len +2;

                    //TransferLen = Neuron(dev, (ExpAppBuffer *)buffer_in, current_src_msg->Len, (ExpAppBuffer *)buffer_out, dev->SubdevNum);

                    dbg("got %zi bytes from Neuron", bytes_current);
                    lvu_debug_data(DEBUG_DUMP3, "I ", bytes_current, buffer_in);

                    if (bytes_current > 0)
                    {
                        lvu_debug_data(DEBUG_DUMP3, "II ", bytes_current +2, buffer_in);

                        for (i = 0; i < MAX_DEVICENODES; i++)
                        {
                            if ((i != dev->SubdevNum) && (0 < dev->udev->pSubDev[i].open_count))
                            {
                                status = qwrite(&dev->udev->pSubDev[i].ReadBufferQueue, buffer_in, bytes_current +2, FALSE);
                                dbg("qwrite()=%d i=%d open_count=%d", status, i, dev->udev->pSubDev[i].open_count);
                            }
                        }
                    }

                    // move packet to transfer buffer
                    memcpy(current_packet + LON_SEND_OFFSET, current_src_msg, current_src_msg->Len +2);
                    dev->send_buffer_used -= current_src_msg->Len +2;

                    // compute crc
                    crc16   = CRC16(current_packet + sizeof(LVU_SEND_HEADER), current_src_msg->Len);
                    pcrc16  = (PUSHORT)(current_packet + current_src_msg->Len + LON_SEND_OFFSET +2);
                    *pcrc16 = NET_SWAB(crc16);

                    bytes_current = current_src_msg->Len + LON_SEND_OFFSET +4;

                    ((PLVU_SEND_HEADER)(current_packet))->Len = cpu_to_le16(bytes_current);

                    // fill header
                    CalculateSendBacklog(dev->udev, current_packet[sizeof(LVU_SEND_HEADER)],  // BYTE lpduHeader,
                        (PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1RecvTicks)),
                        (PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1SendTicks)));

					*(PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1RecvTicks)) = cpu_to_le32( *(PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1RecvTicks)) );
					*(PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1SendTicks)) = cpu_to_le32( *(PULONG)(&(((PLVU_SEND_HEADER)(current_packet))->Beta1SendTicks)) );

                    if (0 < dev->send_buffer_used)
                    {
                        memmove(current_src_msg, current_src_msg + current_src_msg->Len +2, dev->send_buffer_used);
                        //dev->send_buffer_used -= current_src_msg->Len +2;
                    }

                    bytes_written  += bytes_current;
                    current_packet += bytes_current;
                }
                else
                {
                    bytes_current = -1;
                }

                if (0 < bytes_written)
                {
                    lvu_debug_data(DEBUG_DUMP2, ">> ", bytes_written, dev->udev->write_urb->transfer_buffer);

                    // set up our urb  FILL_BULK_URB()
                    usb_fill_bulk_urb(dev->udev->write_urb, dev->udev->usbdev,
                        usb_sndbulkpipe(dev->udev->usbdev, dev->udev->bulk_out_endpointAddr),
                        dev->udev->write_urb->transfer_buffer, bytes_written,
                        lvu_write_bulk_callback, dev->udev);

                    // send the data out the bulk port
                    dev->udev->write_urb->status = 0; // needed!

                    dump_urb(dev->udev->write_urb);

                    //dbg("usb_submit_urb(%d):", dev->SubdevNum);
                    retval = USB_SUBMIT_URB(dev->udev->write_urb, GFP_KERNEL);
                    dbg("usb_submit_urb(%d)=%d", dev->SubdevNum, retval);
                    if (retval)
                    {
                        err("%s() failed submitting write urb, error %d", __FUNCTION__, retval);
						INCR_STATS(dev->udev->Status.base.transaction_timeouts);
						reset_neuron(dev->udev, FALSE, TRUE, WATCHDOG_RESET);

						dev->udev->write_urb->status = 0; // needed!
						dump_urb(dev->udev->write_urb);
						retval = USB_SUBMIT_URB(dev->udev->write_urb, GFP_KERNEL);
						dbg("usb_submit_urb2(%d)=%d", dev->SubdevNum, retval);
						if (retval)
						{
							err("%s(2) failed submitting write urb, error %d", __FUNCTION__, retval);
						}
                        break;
                    }
                    else
                    {
                        INCR_STATS(dev->udev->Status.network.transmitted);
                        retval = bytes_written;
                    }
                }
            }
            else
            {
                if ((current_src_msg->Id != niVNI_BADCRC) && ((current_src_msg->Id & 0xF0) != (niCOMM << 4)))
                {
                    dev->DevStats.write_local++;
                    dbg("got niCOMM message");
                    // move packet to buffer
                    memcpy(buffer_in, current_src_msg, current_src_msg->Len +2);
                    dev->send_buffer_used -= current_src_msg->Len +2;

                    bytes_current = current_src_msg->Len +2;

                    TransferLen = Neuron(dev->udev, (ExpAppBuffer *)buffer_in, current_src_msg->Len+2, (ExpAppBuffer *)buffer_out, dev->SubdevNum);

                    dbg("got %i bytes from Neuron", TransferLen);
                    //lvu_debug_data(DEBUG_DUMP3, "<<<", TransferLen, buffer_out);

                    if (TransferLen > 0)
                    {
                        dev->DevStats.read_local++;
                        //if (pdx->DebugFlag & DEBUG_DUMP2)
                        //  DumpBuffer(DUMP_CHAR_VNI "<< ", &LocalAnswer[LNA_DATA_LEN], TransferLen);

                        //buffer_out[0] = LNA_DATA_LEN + TransferLen;
                        lvu_debug_data(DEBUG_DUMP2, "<< ", TransferLen, buffer_out);

                        status = qwrite(&dev->ReadBufferQueue, buffer_out, TransferLen, FALSE);
                        dbg("qwrite()=%d", status);
                    }

                    if (0 < dev->send_buffer_used)
                    {
                        memmove(current_src_msg, current_src_msg + current_src_msg->Len +2, dev->send_buffer_used);
                    }
                }
                else
                {
                    dbg("got 0x%02X message (dropping)", ((int)(current_src_msg->Id)) & 0xFF);

                    dev->send_buffer_used -= current_src_msg->Len +2;
                    bytes_current = current_src_msg->Len +2;

                    if (0 < dev->send_buffer_used)
                    {
                        memmove(current_src_msg, current_src_msg + current_src_msg->Len +2, dev->send_buffer_used);
                    }
                }
            }
        }
        else
        {
            bytes_current = -1;
        }
    }

    return retval;
}


/**
 *      lvu_ioctl / lvu_unlocked_ioctl
 */
static long lvu_unlocked_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
    int retval;

    switch( cmd )
    {
        char tmpBuf[80];
        case IOCTL_VERSION:
            retval = sprintf(tmpBuf, DRIVER_DESC " " DRIVER_VERSION ", Linux %d.%d.%d",
                (LINUX_VERSION_CODE >> 16) & 0xFF,
                (LINUX_VERSION_CODE >> 8) & 0xFF,
                LINUX_VERSION_CODE & 0xFF ) + 1;
            if (copy_to_user((char *)arg, tmpBuf, retval))
                retval = -EFAULT;
            break;
        default:
            retval = -EINVAL;
    }
    return retval;
}

